@@ -112,7 +112,7 @@ int main(int argc, char *argv[])
     char *ext = strrchr(argv[2], '.');
     if ((ext != NULL) && (strcmp(ext, ".c2") == 0)) {
         int nread = fread(&in_hdr,sizeof(in_hdr),1,fin);
-        assert (nread == sizeof(in_hdr));
+        assert (nread == 1);
         
         if (memcmp(in_hdr.magic, c2_file_magic, sizeof(c2_file_magic)) == 0) {
             fprintf(stderr, "Detected Codec2 file version %d.%d in mode %d\n",
@@ -141,10 +141,6 @@ int main(int argc, char *argv[])
         mode = CODEC2_MODE_1300;
         else if (strcmp(argv[1],"1200") == 0)
         mode = CODEC2_MODE_1200;
-        else if (strcmp(argv[1],"700") == 0)
-        mode = CODEC2_MODE_700;
-        else if (strcmp(argv[1],"700B") == 0)
-        mode = CODEC2_MODE_700B; 
         else if (strcmp(argv[1],"700C") == 0)
         mode = CODEC2_MODE_700C;
         else if (strcmp(argv[1],"450") == 0)
@@ -152,7 +148,7 @@ int main(int argc, char *argv[])
         else if (strcmp(argv[1],"450PWB") == 0)
         mode = CODEC2_MODE_450PWB;
         else {
-        fprintf(stderr, "Error in mode: %s.  Must be 3200, 2400, 1600, 1400, 1300, 1200, 700, 700B, 700C, 450, or 450PWB\n", argv[1]);
+        fprintf(stderr, "Error in mode: %s.  Must be 3200, 2400, 1600, 1400, 1300, 1200, 700C, 450, or 450PWB\n", argv[1]);
         exit(1);
         }
         bit_rate = atoi(argv[1]);
@@ -214,9 +210,6 @@ int main(int argc, char *argv[])
 	    else if (strcmp(long_options[option_index].name, "energy") == 0) {
 	        report_energy = 1;
 	    }
-	    else if (strcmp(long_options[option_index].name, "mlfeat") == 0) {
-	        codec2_open_mlfeat(codec2, optarg);
-	    }
 	    else if (strcmp(long_options[option_index].name, "loadcb") == 0) {
                 /* load VQ stage (700C only) */
                 //fprintf(stderr, "%s\n", optarg+1);
@@ -232,6 +225,10 @@ int main(int argc, char *argv[])
 	    else if (strcmp(long_options[option_index].name, "nopf") == 0) {
 	        codec2_700c_post_filter(codec2, 0);
 	    }
+	    else if (strcmp(long_options[option_index].name, "mlfeat") == 0) {
+		/* dump machine learning features (700C only) */
+		codec2_open_mlfeat(codec2, optarg, NULL);
+	    }
             break;
 
         case 'h':
@@ -366,14 +363,13 @@ int main(int argc, char *argv[])
         if (f_ratek != NULL)
             ret = fread(user_ratek, sizeof(float), K, f_ratek);
         
-	codec2_decode_ber(codec2, buf, bits, ber_est);
- 	fwrite(buf, sizeof(short), nsam, fout);
+    	codec2_decode_ber(codec2, buf, bits, ber_est);
+     	fwrite(buf, sizeof(short), nsam, fout);
 
-	//if this is in a pipeline, we probably don't want the usual
+	    //if this is in a pipeline, we probably don't want the usual
         //buffering to occur
 
         if (fout == stdout) fflush(stdout);
-        if (fin == stdin) fflush(stdin);
 
         if (softdec) {
             ret = (fread(softdec_bits, sizeof(float), nbit, fin) == (size_t)nbit);
@@ -409,7 +405,7 @@ void print_help(const struct option* long_options, int num_opts, char* argv[])
 	int i;
 	char *option_parameters;
 	fprintf(stderr, "\nc2dec - Codec 2 decoder and bit error simulation program\n"
-		"usage: %s 3200|2400|1600|1400|1300|1200|700|700B|700C|450|450PWB InputFile OutputRawFile [OPTIONS]\n\n"
+		"usage: %s 3200|2400|1600|1400|1300|1200|700C|450|450PWB InputFile OutputRawFile [OPTIONS]\n\n"
                 "Options:\n", argv[0]);
         for(i=0; i<num_opts-1; i++) {
 		if(long_options[i].has_arg == no_argument) {
