@@ -46,11 +46,10 @@
 #include "phase.h"
 #include "postfilter.h"
 #include "interp.h"
-#include "ampexp.h"
-#include "phaseexp.h"
 #include "bpf.h"
 #include "bpfb.h"
 #include "newamp1.h"
+#include "lpcnet_freq.h"
 
 void synth_one_frame(int n_samp, codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[], float Pn[], int prede, float *de_mem, float gain);
 void print_help(const struct option *long_options, int num_opts, char* argv[]);
@@ -71,17 +70,12 @@ int main(int argc, char *argv[])
 
     int lpc_model = 0, order = LPC_ORD;
     int lsp = 0, lspd = 0, lspvq = 0;
-    int lspres = 0;
-    int lspjvm = 0, lspjnd = 0, lspmel = 0, lspmelvq = 0;
-    #ifdef __EXPERIMENTAL__
-    int lspanssi = 0,
-    #endif
+    int lspjvm = 0;
     int prede = 0;
     int   postfilt;
-    int   hand_voicing = 0, phaseexp = 0, ampexp = 0, hi = 0, simlpcpf = 0, lspmelread = 0;
+    int   hand_voicing = 0, hi = 0, simlpcpf = 0, modelin=0, modelout=0;
     int   lpcpf = 0;
     FILE *fvoicing = 0;
-    FILE *flspmel = 0;
     int dec;
     int decimate = 1;
     int   amread, Woread, pahw;
@@ -101,39 +95,46 @@ int main(int argc, char *argv[])
     FILE *fjvm = NULL;
     FILE *flspEWov = NULL;
     FILE *ften_ms_centre = NULL;
+    FILE *fmodelout = NULL;
+    FILE *fmodelin = NULL;
     #ifdef DUMP
     int   dump;
     #endif
-    char  ampexp_arg[MAX_STR];
-    char  phaseexp_arg[MAX_STR];
     char  out_file[MAX_STR];
     FILE *fout = NULL;	/* output speech file */
-    int   mel_resampling = 0;
+    int   rateK = 0, newamp1vq = 0, rate_K_dec = 0, perframe=0;
+    int   bands = 0, bands_lower_en;
+    float bands_lower = -1E32;
     int   K = 20;
     float framelength_s = N_S;
-    int   lspEWov = 0;
+    int   lspEWov = 0, rateKWov = 0, first = 0;
+    FILE  *frateKWov = NULL;
     int   ten_ms_centre = 0;
+    FILE  *fphasenn = NULL;
+    FILE  *frateK = NULL;
+    FILE  *frateKin = NULL;
+    int   rateKout, rateKin;
+    FILE *fbands = NULL;
+    int   bands_resample = 0;
     
     char* opt_string = "ho:";
     struct option long_options[] = {
         { "Fs", required_argument, &set_fs, 1 },
-        { "mel", required_argument, &mel_resampling, 1 },
+        { "rateK", no_argument, &rateK, 1 },
+        { "perframe", no_argument, &perframe, 1 },
+        { "newamp1vq", no_argument, &newamp1vq, 1 },
+        { "rateKdec", required_argument, &rate_K_dec, 1 },
+        { "rateKout", required_argument, &rateKout, 1 },
+        { "rateKin", required_argument, &rateKin, 1 },
+        { "bands",required_argument, &bands, 1 },
+        { "bands_lower",required_argument, &bands_lower_en, 1 },
+        { "bands_resample", no_argument, &bands_resample, 1 },
         { "lpc", required_argument, &lpc_model, 1 },
-        { "lspjnd", no_argument, &lspjnd, 1 },
-        { "lspmel", no_argument, &lspmel, 1 },
-        { "lspmelread", required_argument, &lspmelread, 1 },
-        { "lspmelvq", no_argument, &lspmelvq, 1 },
         { "lsp", no_argument, &lsp, 1 },
         { "lspd", no_argument, &lspd, 1 },
         { "lspvq", no_argument, &lspvq, 1 },
-        { "lspres", no_argument, &lspres, 1 },
         { "lspjvm", no_argument, &lspjvm, 1 },
-        #ifdef __EXPERIMENTAL__
-        { "lspanssi", no_argument, &lspanssi, 1 },
-        #endif
         { "phase0", no_argument, &phase0, 1 },
-        { "phaseexp", required_argument, &phaseexp, 1 },
-        { "ampexp", required_argument, &ampexp, 1 },
         { "postfilter", no_argument, &postfilt, 1 },
         { "hand_voicing", required_argument, &hand_voicing, 1 },
         { "dec", required_argument, &dec, 1 },
@@ -155,8 +156,12 @@ int main(int argc, char *argv[])
         { "Woread", required_argument, &Woread, 1 },
         { "pahw", required_argument, &pahw, 1 },
         { "lspEWov", required_argument, &lspEWov, 1 },
+        { "rateKWov", required_argument, &rateKWov, 1 },
+        { "first", no_argument, &first, 1 },
         { "ten_ms_centre", required_argument, &ten_ms_centre, 1 },
         { "framelength_s", required_argument, NULL, 0 },
+        { "modelout",  required_argument, &modelout, 1 },
+        { "modelin",   required_argument, &modelin, 1 },
         #ifdef DUMP
         { "dump", required_argument, &dump, 1 },
         #endif
@@ -189,8 +194,6 @@ int main(int argc, char *argv[])
                     fprintf(stderr, "Error Fs must be 8000 or 16000\n");
                     exit(1);
                 }
-            } else if(strcmp(long_options[option_index].name, "mel") == 0) {
-                K = atoi(optarg);
             } else if(strcmp(long_options[option_index].name, "lpc") == 0) {
                 order = atoi(optarg);
             #ifdef DUMP
@@ -202,6 +205,35 @@ int main(int argc, char *argv[])
                   || strcmp(long_options[option_index].name, "lspd") == 0
                   || strcmp(long_options[option_index].name, "lspvq") == 0) {
 	        assert(order == LPC_ORD);
+            } else if(strcmp(long_options[option_index].name, "rateKdec") == 0) {
+                rate_K_dec = atoi(optarg);
+                fprintf(stderr, "rate_K_dec: %d\n", rate_K_dec);
+	    } else if(strcmp(long_options[option_index].name, "rateKout") == 0) {
+                /* read model records from file or stdin */
+                if ((frateK = fopen(optarg,"wb")) == NULL) {
+	            fprintf(stderr, "Error opening output rateK file: %s: %s\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
+                fprintf(stderr, "each record is %d bytes\n", (int)(K*sizeof(float)));
+	    } else if(strcmp(long_options[option_index].name, "rateKin") == 0) {
+                /* read model records from file or stdin */
+                if ((frateKin = fopen(optarg,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening input rateK file: %s: %s\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
+                fprintf(stderr, "each record is %d bytes\n", (int)(K*sizeof(float)));
+	    } else if(strcmp(long_options[option_index].name, "bands") == 0) {
+                /* write mel spaced band energies to file or stdout */
+                if ((fbands = fopen(optarg,"wb")) == NULL) {
+	            fprintf(stderr, "Error opening bands file: %s: %s\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
+	    } else if(strcmp(long_options[option_index].name, "bands_lower") == 0) {
+		bands_lower = atof(optarg);
+		fprintf(stderr, "bands_lower: %f\n", bands_lower);
             } else if(strcmp(long_options[option_index].name, "dec") == 0) {
 
                 decimate = atoi(optarg);
@@ -211,11 +243,11 @@ int main(int argc, char *argv[])
 	        }
 
                 if (!phase0) {
-                    printf("needs --phase0 to resample phase when using --dec\n");
+                    fprintf(stderr, "needs --phase0 to resample phase when using --dec\n");
                     exit(1);
                 }
                 if (!lpc_model) {
-                    printf("needs --lpc [order] to resample amplitudes when using --dec\n");
+                    fprintf(stderr, "needs --lpc [order] to resample amplitudes when using --dec\n");
                     exit(1);
                 }
 
@@ -225,12 +257,6 @@ int main(int argc, char *argv[])
 		        optarg, strerror(errno));
                     exit(1);
                 }
-            } else if(strcmp(long_options[option_index].name, "lspmelread") == 0) {
-	        if ((flspmel = fopen(optarg,"rb")) == NULL) {
-	            fprintf(stderr, "Error opening float lspmel file: %s: %s.\n",
-		        optarg, strerror(errno));
-                    exit(1);
-                }
             } else if(strcmp(long_options[option_index].name, "Woread") == 0) {
 	        if ((fWo = fopen(optarg,"rb")) == NULL) {
 	            fprintf(stderr, "Error opening float Wo file: %s: %s.\n",
@@ -261,10 +287,6 @@ int main(int argc, char *argv[])
 		        optarg, strerror(errno));
                     exit(1);
                 }
-	    } else if(strcmp(long_options[option_index].name, "phaseexp") == 0) {
-		strcpy(phaseexp_arg, optarg);
-	    } else if(strcmp(long_options[option_index].name, "ampexp") == 0) {
-		strcpy(ampexp_arg, optarg);
 	    } else if(strcmp(long_options[option_index].name, "gain") == 0) {
 		gain = atof(optarg);
 	    } else if(strcmp(long_options[option_index].name, "framelength_s") == 0) {
@@ -303,17 +325,45 @@ int main(int argc, char *argv[])
 	            fprintf(stderr, "Error opening lspEWov float file: %s: %s\n",
 		        optarg, strerror(errno));
                     exit(1);
-                }                 
+                }
+	    } else if(strcmp(long_options[option_index].name, "rateKWov") == 0) {
+                /* feature file for deep learning experiments */
+                rateK = 1; newamp1vq = 1;
+	        if ((frateKWov = fopen(optarg,"wb")) == NULL) {
+	            fprintf(stderr, "Error opening rateKWov float file: %s: %s\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
 	    } else if(strcmp(long_options[option_index].name, "ten_ms_centre") == 0) {
-                /* dump 10ms of audio centred on analysis frame to check time alignment with 
+                /* dump 10ms of audio centred on analysis frame to check time alignment with
                    16 kHz source audio */
                 ten_ms_centre = 1;
 	        if ((ften_ms_centre = fopen(optarg,"wb")) == NULL) {
 	            fprintf(stderr, "Error opening ten_ms_centre short file: %s: %s\n",
 		        optarg, strerror(errno));
                     exit(1);
-                }                 
-	    } else if(strcmp(long_options[option_index].name, "rate") == 0) {
+                }
+	    } else if(strcmp(long_options[option_index].name, "modelout") == 0) {
+                /* write model records to file or stdout */
+                modelout = 1;
+                if (strcmp(optarg, "-") == 0) fmodelout = stdout;
+	        else if ((fmodelout = fopen(optarg,"wb")) == NULL) {
+	            fprintf(stderr, "Error opening modelout file: %s: %s\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
+                fprintf(stderr, "each model record is %d bytes\n", (int)sizeof(MODEL));
+	    } else if(strcmp(long_options[option_index].name, "modelin") == 0) {
+                /* read model records from file or stdin */
+                modelin = 1;
+                if (strcmp(optarg, "-") == 0) fmodelin = stdin;
+	        else if ((fmodelin = fopen(optarg,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening modelin file: %s: %s\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
+                fprintf(stderr, "each model record is %d bytes\n", (int)sizeof(MODEL));
+            } else if(strcmp(long_options[option_index].name, "rate") == 0) {
                 if(strcmp(optarg,"3200") == 0) {
 	            lpc_model = 1;
 		    scalar_quant_Wo_e = 1;
@@ -362,8 +412,8 @@ int main(int argc, char *argv[])
             break;
 
          case 'h':
-            print_help(long_options, num_opts, argv);
-            break;
+             print_help(long_options, num_opts, argv);
+             break;
 
          case 'o':
 	     if (strcmp(optarg, "-") == 0) fout = stdout;
@@ -443,12 +493,11 @@ int main(int argc, char *argv[])
     struct AEXP *aexp = NULL;
     #endif
     float bpf_buf[BPF_N+N_SAMP];
-    float lspmelvq_mse = 0.0;
 
     COMP Aw[FFT_ENC];
     COMP H[MAX_AMP];
 
- 
+
     for(i=0; i<m_pitch; i++) {
 	Sn[i] = 1.0;
 	Sn_pre[i] = 1.0;
@@ -476,12 +525,6 @@ int main(int argc, char *argv[])
     ex_phase[0] = 0;
     Woe_[0] = Woe_[1] = 1.0;
 
-    /*
-      printf("lspd: %d lspdt: %d lspdt_mode: %d  phase0: %d postfilt: %d "
-	   "decimate: %d dt: %d\n",lspd,lspdt,lspdt_mode,phase0,postfilt,
-	   decimate,dt);
-    */
-
     /* Initialise ------------------------------------------------------------*/
 
     fft_fwd_cfg = codec2_fft_alloc(FFT_ENC, 0, NULL, NULL);   /* fwd FFT,used in several places   */
@@ -492,15 +535,6 @@ int main(int argc, char *argv[])
 
     make_analysis_window(&c2const, fft_fwd_cfg, w, W);
     make_synthesis_window(&c2const, Pn);
-    quantise_init();
-
-    /* disabled for now while we convert to runtime n_samp */
-    #if 0
-    if (phaseexp)
-	pexp = phase_experiment_create();
-    if (ampexp)
-	aexp = amp_experiment_create();
-    #endif
 
     if (bpfb_en)
         bpf_en = 1;
@@ -521,14 +555,26 @@ int main(int argc, char *argv[])
 
     /* mel resampling experiments */
 
-    float rate_K_sample_freqs_kHz[K];
-    if (mel_resampling) {
-        float mel_start = ftomel(100); 
-        float mel_end = ftomel(0.95*Fs/2);
-        mel_sample_freqs_kHz(rate_K_sample_freqs_kHz, K, mel_start, mel_end);
-        //for(i=0; i<K; i++)
-        //    fprintf(stderr, "%d %f\n", i, rate_K_sample_freqs_kHz[i]);
+    float rate_K_sample_freqs_kHz[K]; float se = 0.0; int nse = 0;
+    if (rateK) {
+	mel_sample_freqs_kHz(rate_K_sample_freqs_kHz, NEWAMP1_K, ftomel(200.0), ftomel(3700.0) );
     }
+    float rate_K_vec_delay[rate_K_dec+1][K];
+    float rate_K_vec_delay_[rate_K_dec+1][K];
+    MODEL rate_K_model_delay[rate_K_dec+1];
+    for (int d=0; d<=rate_K_dec; d++) {
+        for(int k=0; k<K; k++) {
+            rate_K_vec_delay[d][k] = 0;
+            rate_K_vec_delay_[d][k] = 0;
+        }
+        for(m=1; m<=MAX_AMP; m++)
+            rate_K_model_delay[d].A[m] = 0.0;
+        rate_K_model_delay[d].Wo = c2const.Wo_min;
+        rate_K_model_delay[d].L = M_PI/prev_model_dec.Wo;
+        rate_K_model_delay[d].voiced = 0;
+    }
+    float eq[K];
+    for(int k=0; k<K; k++) eq[k] = 0;
 
     /*----------------------------------------------------------------* \
 
@@ -595,20 +641,17 @@ int main(int argc, char *argv[])
         dump_Sn(m_pitch, Sn); dump_Sw(Sw); dump_model(&model);
         #endif
 
-        #if 0
-	if (ampexp)
-	    amp_experiment(aexp, &model, ampexp_arg);
+	/* speech centred on analysis frame for Deep Learning work */
 
-	if (phaseexp) {
-            #ifdef DUMP
-	    dump_phase(&model.phi[0], model.L);
-            #endif
-	    phase_experiment(pexp, &model, phaseexp_arg);
-            #ifdef DUMP
-	    dump_phase_(&model.phi[0], model.L);
-            #endif
+	if (ten_ms_centre) {
+	    int n_10_ms = Fs*0.01;
+	    int n_5_ms = Fs*0.005;
+	    short buf[n_10_ms];
+	    for(i=0; i<n_10_ms; i++) {
+		buf[i] = Sn[m_pitch/2-n_5_ms+i];
+	    }
+	    fwrite(buf, n_10_ms, sizeof(short), ften_ms_centre);
 	}
-        #endif
 
 	if (hi) {
 	    int m;
@@ -618,45 +661,25 @@ int main(int argc, char *argv[])
 		model.A[m] = 0.0;
 	}
 
-	/*------------------------------------------------------------*\
-
-	                     Mel scale resampling
-
-	\*------------------------------------------------------------*/
-
-        if (mel_resampling) {
-            float rate_K_vec[K];
-            resample_const_rate_f(&c2const, &model, rate_K_vec, rate_K_sample_freqs_kHz, K);
-            resample_rate_L(&c2const, &model, rate_K_vec, rate_K_sample_freqs_kHz, K);
-        }
-
 	/*------------------------------------------------------------*\
 
                             Zero-phase modelling
 
 	\*------------------------------------------------------------*/
 
+	/* estimate voicing - do this all the time so model.voicing
+	 * is set, useful for machine learning work */
+	snr = est_voicing_mbe(&c2const, &model, Sw, W);
+
 	if (phase0) {
             #ifdef DUMP
 	    dump_phase(&model.phi[0], model.L);
             #endif
 
-	    /* determine voicing */
-
-	    #if 0
-            snr = est_voicing_mbe(&c2const, &model, Sw, W, Sw_, Ew);
-            #else
-	    snr = est_voicing_mbe(&c2const, &model, Sw, W);
-            #endif
-
 	    if (dump_pitch_e)
 		fprintf(fjvm, "%f %f %d ", model.Wo, snr, model.voiced);
 
-	    //printf("snr %3.2f v: %d Wo: %f prev_Wo: %f\n", snr, model.voiced,
-	    //	   model.Wo, prev_uq_Wo);
             #ifdef DUMP
-	    //dump_Sw_(Sw_);
-	    //dump_Ew(Ew);
 	    dump_snr(snr);
             #endif
 
@@ -682,42 +705,15 @@ int main(int argc, char *argv[])
             e = speech_to_uq_lsps(lsps, ak, Sn, w, m_pitch, order);
             for(i=0; i<order; i++)
                 lsps_[i] = lsps[i];
-            
+
             #ifdef DUMP
 	    dump_ak(ak, order);
             dump_E(e);
             #endif
 
-	    /* tracking down -ve energy values with BW expansion */
-	    /*
-	    if (e < 0.0) {
-		int i;
-		FILE*f=fopen("x.txt","wt");
-		for(i=0; i<M_PITCH; i++)
-		    fprintf(f,"%f\n", Sn[i]);
-		fclose(f);
-		printf("e = %f frames = %d\n", e, frames);
-		for(i=0; i<order; i++)
-		    printf("%f ", ak[i]);
-		exit(0);
-	    }
-	    */
-
 	    if (dump_pitch_e)
 		fprintf(fjvm, "%f\n", e);
 
-            /* speech centred on analysis frame for Deep Learning work */
-            
-            if (ten_ms_centre) {
-                int n_10_ms = Fs*0.01;
-                int n_5_ms = Fs*0.005;
-                short buf[n_10_ms];
-                for(i=0; i<n_10_ms; i++) {
-                    buf[i] = Sn[m_pitch/2-n_5_ms+i];
-                }
-                fwrite(buf, n_10_ms, sizeof(short), ften_ms_centre);                  
-            }
-            
             #ifdef DUMP
             dump_lsp(lsps);
             #endif
@@ -737,14 +733,6 @@ int main(int argc, char *argv[])
 		lsp_to_lpc(lsps_, ak, LPC_ORD);
 	    }
 
-#ifdef __EXPERIMENTAL__
-	    if (lspvq) {
-		lspvq_quantise(lsps, lsps_, LPC_ORD);
-		bw_expand_lsps(lsps_, LPC_ORD, 50.0, 100.0);
-		lsp_to_lpc(lsps_, ak, LPC_ORD);
-	    }
-#endif
-
 	    if (lspjvm) {
 		/* Jean-Marc's multi-stage, split VQ */
 		lspjvm_quantise(lsps, lsps_, LPC_ORD);
@@ -756,101 +744,7 @@ int main(int argc, char *argv[])
 		}
 	    }
 
-#ifdef __EXPERIMENTAL__
-	    if (lspanssi) {
-		/*  multi-stage VQ from Anssi Ramo OH3GDD */
-
-		lspanssi_quantise(lsps, lsps_, LPC_ORD, 5);
-		bw_expand_lsps(lsps_, LPC_ORD, 50.0, 100.0);
-		lsp_to_lpc(lsps_, ak, LPC_ORD);
-	    }
-#endif
-
-	    /* experimenting with non-linear LSP spacing to see if
-	       it's just noticable */
-
-	    if (lspjnd) {
-		for(i=0; i<LPC_ORD; i++)
-		    lsps_[i] = lsps[i];
-		locate_lsps_jnd_steps(lsps_, LPC_ORD);
-		lsp_to_lpc(lsps_, ak, LPC_ORD);
-	    }
-
-	    /* Another experiment with non-linear LSP spacing, this
-	       time using a scaled version of mel frequency axis
-	       warping.  The scaling is such that the integer output
-	       can be directly sent over the channel.
-	    */
-
-	    if (lspmel) {
-		float f, f_;
-		float mel[order];
-		int   mel_indexes[order];
-
-		for(i=0; i<order; i++) {
-		    f = (4000.0/PI)*lsps[i];
-		    mel[i] = floor(2595.0*log10(1.0 + f/700.0) + 0.5);
-		}
-
-                #define MEL_ROUND 25
-		for(i=1; i<order; i++) {
-		    if (mel[i] <= mel[i-1]+MEL_ROUND) {
-			mel[i]+=MEL_ROUND/2;
-			mel[i-1]-=MEL_ROUND/2;
-                        i = 1;
-                    }
-		}
-
-                #ifdef DUMP
-                dump_mel(mel, order);
-                #endif
-
- 		encode_mels_scalar(mel_indexes, mel, 6);
-                #ifdef DUMP
-                dump_mel_indexes(mel_indexes, 6);
-                #endif
-		//decode_mels_scalar(mel, mel_indexes, 6);
-
-                /* read in VQed lsp-mels from octave/melvq.m */
-
-                if (lspmelread) {
-                    float mel_[order];
-                    int ret = fread(mel_, sizeof(float), order, flspmel);
-                    assert(ret == order);
-                    for(i=0; i<order; i++) {
-                        lspmelvq_mse += pow(mel[i] - mel_[i], 2.0);
-                        mel[i] = mel_[i];
-                    }
-                }
-
-                if (lspmelvq) {
-                    int indexes[3];
-                    //lspmelvq_mse += lspmelvq_quantise(mel, mel, order);
-                    lspmelvq_mse += lspmelvq_mbest_encode(indexes, mel, mel, order, 5);
-                }
-
-                /* ensure no unstable filters after quantisation */
-
-                #define MEL_ROUND 25
-		for(i=1; i<order; i++) {
-		    if (mel[i] <= mel[i-1]+MEL_ROUND) {
-			mel[i]+=MEL_ROUND/2;
-			mel[i-1]-=MEL_ROUND/2;
-                        i = 1;
-                    }
-		}
-
-		for(i=0; i<order; i++) {
-		    f_ = 700.0*( pow(10.0, mel[i]/2595.0) - 1.0);
-		    lsps_[i] = f_*(PI/4000.0);
-		}
-
-		lsp_to_lpc(lsps_, ak, order);
-
-	    }
-
 	    if (scalar_quant_Wo_e) {
-
 		e = decode_energy(encode_energy(e, E_BITS), E_BITS);
                 model.Wo = decode_Wo(&c2const, encode_Wo(&c2const, model.Wo, WO_BITS), WO_BITS);
 		model.L  = PI/model.Wo; /* if we quantise Wo re-compute L */
@@ -864,9 +758,7 @@ int main(int argc, char *argv[])
 	    }
 
 	    if (vector_quant_Wo_e) {
-
 		/* JVM's experimental joint Wo & LPC energy quantiser */
-
 		quantise_WoE(&c2const, &model, &e, Woe_);
 	    }
 
@@ -884,21 +776,181 @@ int main(int argc, char *argv[])
         }
 
         /* dump features for Deep learning, placed here so we can get quantised features */
-        
+
         if (lspEWov) {
-            /* order LSPs - energy - Wo - voicing flag - order LPCs */                
+            /* order LSPs - energy - Wo - voicing flag - order LPCs */
             if (lsp)
                 fwrite(lsps_, order, sizeof(float), flspEWov);
             else
                 fwrite(lsps, order, sizeof(float), flspEWov);
-                    
+
             fwrite(&e, 1, sizeof(float), flspEWov);
-            fwrite(&model.Wo, 1, sizeof(float), flspEWov); 
+            fwrite(&model.Wo, 1, sizeof(float), flspEWov);
             float voiced_float = model.voiced;
             fwrite(&voiced_float, 1, sizeof(float), flspEWov);
             fwrite(&ak[1], order, sizeof(float), flspEWov);
         }
-            
+
+	/* LPCNet type mel spaced band ML data */
+	float bands_mean = 0.0;
+	if (fbands) {
+	    float bandE[LPCNET_FREQ_MAX_BANDS];
+            float freqkHz[LPCNET_FREQ_MAX_BANDS];
+	    int nbands = lpcnet_compute_band_energy(bandE, freqkHz, Sw, Fs, FFT_ENC);
+	    for(int i=0; i<nbands; i++)
+		bands_mean += bandE[i];
+	    bands_mean /= nbands;
+	    //fprintf(stderr, "bands_mean: %f bands_lower %f\n", bands_mean,  bands_lower);
+	    if (bands_mean > bands_lower)
+ 		assert(fwrite(bandE, sizeof(float), nbands, fbands) == nbands);
+            // optionally reconstruct [Am} by linear interpolation of band energies,
+            // this doesn't sound very Good
+            if (bands_resample)
+                resample_rate_L(&c2const, &model, &bandE[1], &freqkHz[1], nbands-2);
+	}
+
+	/*------------------------------------------------------------*\
+
+	            Optional newamp1 simulation, as used in 700C
+
+	\*------------------------------------------------------------*/
+
+        if (rateK) {
+            float rate_K_vec[K];
+            resample_const_rate_f(&c2const, &model, rate_K_vec, rate_K_sample_freqs_kHz, K);
+
+	    if (frateK != NULL)
+		assert(fwrite(rate_K_vec, sizeof(float), K, frateK) == K);
+	    
+	    if (frateKin != NULL) {
+		assert(fread(rate_K_vec, sizeof(float), K, frateKin) == K);
+		/* apply newamp1 postfilter - this helped male samples with VQVAE work */
+                float sum = 0.0;
+                for(int k=0; k<K; k++)
+                    sum += rate_K_vec[k];
+                float mean = sum/K;
+                float rate_K_vec_no_mean[K];
+                for(int k=0; k<K; k++)
+                    rate_K_vec_no_mean[k] = rate_K_vec[k] - mean;
+		post_filter_newamp1(rate_K_vec_no_mean,  rate_K_sample_freqs_kHz, K, 1.5);
+                for(int k=0; k<K; k++)
+                    rate_K_vec[k] = rate_K_vec_no_mean[k] +  mean;
+	    }
+	    
+            float rate_K_vec_[K];
+            if (newamp1vq) {
+                /* remove mean */
+                float sum = 0.0;
+                for(int k=0; k<K; k++)
+                    sum += rate_K_vec[k];
+                float mean = sum/K;
+                float rate_K_vec_no_mean[K];
+                for(int k=0; k<K; k++)
+                    rate_K_vec_no_mean[k] = rate_K_vec[k] - mean;
+
+		newamp1_eq(rate_K_vec_no_mean, eq, K, 1);
+
+                /* two stage VQ */
+                float rate_K_vec_no_mean_[K]; int indexes[2];
+                rate_K_mbest_encode(indexes, rate_K_vec_no_mean, rate_K_vec_no_mean_, K, NEWAMP1_VQ_MBEST_DEPTH);
+                for(int k=0; k<K; k++)
+                    rate_K_vec_[k] = rate_K_vec_no_mean_[k] + mean;
+
+                /* running sum of squared error for variance calculation */
+                for(int k=0; k<K; k++)
+                    se += pow(rate_K_vec_no_mean[k]-rate_K_vec_no_mean_[k],2.0);
+                nse += K;
+            }
+            else {
+                for(int k=0; k<K; k++)
+                    rate_K_vec_[k] = rate_K_vec[k];
+            }
+
+	    if (frateKWov != NULL) {
+		/* We use standard nb_features=55 feature records for compatability with train_lpcnet.py */
+		float features[55] = {0};
+		/* just using 18/20 for compatability with LPCNet, coarse scaling for NN imput */
+		for(int i=0; i<18; i++)
+		    features[i] = (rate_K_vec_[i]-30)/40;
+		// keep in range of 40 ... 255 for pitch embedding
+		int pitch_index = 21 + 2.0*M_PI/model.Wo;
+		features[36] = 0.02*(pitch_index-100);
+		//features[36] = (model.Wo - c2const.Wo_min)/(c2const.Wo_max - c2const.Wo_min) - 0.5;
+		features[37] = model.voiced;
+		if (first)
+		    features[18] = -0.9;
+		if (lpc_model) {
+		    MODEL model_;
+		    model_.Wo = model.Wo;
+		    model_.L  = model.L;
+		    model_.voiced = model.voiced;
+		    float Rk[order+1], ak[order+1];
+		    resample_rate_L(&c2const, &model_, rate_K_vec_, rate_K_sample_freqs_kHz, K);
+		    determine_autoc(&c2const, Rk, order, &model_, NEWAMP1_PHASE_NFFT, phase_fft_fwd_cfg, phase_fft_inv_cfg);
+		    /* -40 dB noise floor and Lag windowing from LPCNet/freq.c - helps reduce large spikes in spectrum when LPC
+                       analysis loses it. */
+		    Rk[0] += Rk[0]*1e-4 + 320/12/38.;
+		    for (i=1;i<order+1;i++) Rk[i] *= (1 - 6e-5*i*i);
+		    levinson_durbin(Rk, ak, order);
+
+		    for(int i=0; i<order; i++)
+			features[18+i] = ak[i+1];
+		}
+		fwrite(features, 55, sizeof(float), frateKWov);
+	    }
+
+            if (rate_K_dec) {
+                // update delay lines
+                for(int d=0; d<rate_K_dec; d++) {
+                    rate_K_model_delay[d] = rate_K_model_delay[d+1];
+                    memcpy(&rate_K_vec_delay[d][0], &rate_K_vec_delay[d+1][0], sizeof(float)*K);
+                }
+                rate_K_model_delay[rate_K_dec] = model;
+                memcpy(&rate_K_vec_delay[rate_K_dec][0], rate_K_vec_, sizeof(float)*K);
+
+                if ((frames % rate_K_dec) == 0) {
+                    // every rate_K_dec frames, calculate interpolated output values
+                    if (perframe) {
+                        // calculate interpolation coeff c for each frame
+                        float *A = &rate_K_vec_delay[0][0];
+                        float *B = &rate_K_vec_delay[rate_K_dec][0];
+                        for(int d=0; d<=rate_K_dec; d++) {
+                            float *T = &rate_K_vec_delay[d][0];
+                            float num = 0.0, den = 0.0;
+                            for(int k=0; k<K; k++) {
+                                num += (B[k]-T[k])*(A[k]-B[k]);
+                                den += (A[k]-B[k])*(A[k]-B[k]);
+                            }
+                            float c = -num/den;
+                            for(int k=0; k<K; k++)
+                                rate_K_vec_delay_[d][k] = c*A[k] + (1.0-c)*B[k];
+                        }
+                    }
+                    else {
+                        // use linear interpolation
+                        float c=0.0, inc = 1.0/rate_K_dec;
+                        for(int d=0; d<=rate_K_dec; d++) {
+                            for(int k=0; k<K; k++)
+                                rate_K_vec_delay_[d][k] = (1.0-c)*rate_K_vec_delay[0][k] + c*rate_K_vec_delay[rate_K_dec][k];
+                            c += inc;
+                        }
+                    }
+                } else {
+                    // otherwise just shift out frames we have already interpolated
+                    for(int d=0; d<rate_K_dec; d++) {
+                        memcpy(&rate_K_vec_delay_[d][0], &rate_K_vec_delay_[d+1][0], sizeof(float)*K);
+                    }
+                }
+
+                // output from delay line
+                model = rate_K_model_delay[0];
+                for(int k=0; k<K; k++)
+                    rate_K_vec_[k] = rate_K_vec_delay_[0][k];
+            }
+	    
+            resample_rate_L(&c2const, &model, rate_K_vec_, rate_K_sample_freqs_kHz, K);
+        }
+
 	/*------------------------------------------------------------*\
 
           Synthesise and optional decimation to 20 or 40ms frame rate
@@ -951,6 +1003,14 @@ int main(int argc, char *argv[])
                     #endif
                 }
 
+                if (modelin) {
+                    int nrec;
+                    nrec = fread(&model_dec[i],sizeof(MODEL),1,fmodelin);
+                    if (nrec != 1) {
+			fprintf(stderr, "Warning - error reading model in record in frame %d - do you have enough records in file?\n", frames);
+		    }
+                }
+
                 if (phase0) {
                     /* optionally read in Aw, replacing values generated using LPC */
 
@@ -958,7 +1018,7 @@ int main(int argc, char *argv[])
                         int ret = fread(Aw, sizeof(COMP), FFT_ENC, faw);
                         assert(ret == FFT_ENC);
                     }
-                    
+
                     /* optionally read in Hm directly, bypassing sampling of Aw[] */
 
                     if (hmread) {
@@ -970,11 +1030,20 @@ int main(int argc, char *argv[])
                     phase_synth_zero_order(n_samp, &model_dec[i], ex_phase, H);
                 }
 
-                
                 if (postfilt)
                     postfilter(&model_dec[i], &bg_est);
                 synth_one_frame(n_samp, fftr_inv_cfg, buf, &model_dec[i], Sn_, Pn, prede, &de_mem, gain);
-                if (fout != NULL) fwrite(buf,sizeof(short),N_SAMP,fout);
+                if (fout != NULL)
+                    fwrite(buf,sizeof(short),N_SAMP,fout);
+                if (modelout) {
+		    /* optionally don't write to filter out low energy frames */
+		    if (bands) {
+			if (bands_mean > bands_lower)
+			    fwrite(&model_dec[i],sizeof(MODEL),1,fmodelout);
+		    }
+		    else
+			fwrite(&model_dec[i],sizeof(MODEL),1,fmodelout);
+		}
             }
 
             /* update memories for next frame ----------------------------*/
@@ -1000,16 +1069,10 @@ int main(int argc, char *argv[])
 
     if (lpc_model) {
     	fprintf(stderr, "SNR av = %5.2f dB\n", sum_snr/frames);
-        if (lspmelvq || lspmelread)
-            fprintf(stderr, "lspmelvq std = %3.1f Hz\n", sqrt(lspmelvq_mse/frames));
     }
-
-    #if 0
-    if (phaseexp)
-	phase_experiment_destroy(pexp);
-    if (ampexp)
-	amp_experiment_destroy(aexp);
-    #endif
+    if (newamp1vq) {
+    	fprintf(stderr, "var: %3.2f dB*dB\n", se/nse);
+    }
     #ifdef DUMP
     if (dump)
 	dump_off();
@@ -1020,15 +1083,20 @@ int main(int argc, char *argv[])
 
     nlp_destroy(nlp_states);
 
-    if (flspmel != NULL) fclose(flspmel);
     if (fam     != NULL) fclose(fam);
     if (fWo     != NULL) fclose(fWo);
     if (faw     != NULL) fclose(faw);
     if (fhm     != NULL) fclose(fhm);
     if (fjvm    != NULL) fclose(fjvm);
     if (flspEWov != NULL) fclose(flspEWov);
+    if (fphasenn != NULL) fclose(fphasenn);
+    if (frateK != NULL) fclose(frateK);
+    if (frateKin != NULL) fclose(frateKin);
     if (ften_ms_centre != NULL) fclose(ften_ms_centre);
-    
+    if (fmodelout != NULL) fclose(fmodelout);
+    if (fbands != NULL) fclose(fbands);
+    if (frateKWov != NULL) fclose(frateKWov);
+
     return 0;
 }
 
@@ -1086,4 +1154,3 @@ void print_help(const struct option* long_options, int num_opts, char* argv[])
 
 	exit(1);
 }
-
