@@ -24,6 +24,7 @@
 */
 
 // Low pass filter, sample rate 8000 hz, 0.2 dB ripple, 100 dB atten, pass 550 hz, stop 750 hz.
+// Used to build 700D BPF
 float filtP550S750[160]={
  0.000001500540125945, 0.000020553368071006, 0.000052842049763802, 0.000112071233638701,
  0.000202565299657164, 0.000325476960438197, 0.000474396686568771, 0.000633746562372497, 0.000778858561033731,
@@ -60,9 +61,349 @@ float filtP550S750[160]={
  0.000001500540125945
 };
 
+
+/*
+  Low pass filter, sample rate 8000 Hz, 60dB dB atten, pass 400 Hz, stop 600 Hz
+  Used for datac0/datac1
+  Generated using Octave:
+ 
+  octave:170> h = fir1(99, 1000/8000);
+  octave:171> save_array_c_header(h,"filtP400S600","t.h")
+*/
+
+float filtP400S600[100]={
+   0.0002190442859529,
+   0.0000253865765807,
+  -0.0001891607957119,
+  -0.0004060215259363,
+  -0.0006000085594001,
+  -0.0007388342366621,
+  -0.0007857143550906,
+  -0.0007055101203620,
+  -0.0004737796456378,
+  -0.0000870157214033,
+   0.0004284822493668,
+   0.0010119603553043,
+   0.0015702670713406,
+   0.0019880170510650,
+   0.0021456291121880,
+   0.0019431076070540,
+   0.0013256670818267,
+   0.0003060762051584,
+  -0.0010217279135397,
+  -0.0024824214174461,
+  -0.0038352454444032,
+  -0.0048061266813482,
+  -0.0051327431531073,
+  -0.0046162222656579,
+  -0.0031708262919099,
+  -0.0008619578234589,
+   0.0020765315052916,
+   0.0052523330844492,
+   0.0081546351293169,
+   0.0102217640867241,
+   0.0109290873716773,
+   0.0098852729183138,
+   0.0069220529480671,
+   0.0021618869962732,
+  -0.0039502956265373,
+  -0.0106608379232220,
+  -0.0169699130332444,
+  -0.0217388847493965,
+  -0.0238311162893102,
+  -0.0222687242579450,
+  -0.0163838891184744,
+  -0.0059424763426467,
+   0.0087797880290842,
+   0.0269827101964810,
+   0.0473946488239843,
+   0.0683898571372469,
+   0.0881575538806286,
+   0.1049023332507096,
+   0.1170504418131305,
+   0.1234349185246360,
+   0.1234349185246360,
+   0.1170504418131306,
+   0.1049023332507097,
+   0.0881575538806286,
+   0.0683898571372469,
+   0.0473946488239843,
+   0.0269827101964810,
+   0.0087797880290842,
+  -0.0059424763426467,
+  -0.0163838891184744,
+  -0.0222687242579450,
+  -0.0238311162893102,
+  -0.0217388847493965,
+  -0.0169699130332444,
+  -0.0106608379232220,
+  -0.0039502956265373,
+   0.0021618869962732,
+   0.0069220529480671,
+   0.0098852729183138,
+   0.0109290873716773,
+   0.0102217640867241,
+   0.0081546351293169,
+   0.0052523330844492,
+   0.0020765315052916,
+  -0.0008619578234589,
+  -0.0031708262919099,
+  -0.0046162222656579,
+  -0.0051327431531073,
+  -0.0048061266813482,
+  -0.0038352454444032,
+  -0.0024824214174461,
+  -0.0010217279135397,
+   0.0003060762051584,
+   0.0013256670818267,
+   0.0019431076070540,
+   0.0021456291121880,
+   0.0019880170510650,
+   0.0015702670713406,
+   0.0010119603553043,
+   0.0004284822493668,
+  -0.0000870157214033,
+  -0.0004737796456378,
+  -0.0007055101203620,
+  -0.0007857143550906,
+  -0.0007388342366621,
+  -0.0006000085594001,
+  -0.0004060215259363,
+  -0.0001891607957119,
+   0.0000253865765807,
+   0.0002190442859529
+};
+
+
+/*
+  Low pass filter, sample rate 8000 Hz, 60dB dB atten, pass 650 Hz, stop 900 Hz
+  Alternate filter for use with 700D and clipper.  A litte extra bandwidth helps
+  reduce PAPR.
+
+  Generated using Octave:
+  octave:170> h = fir1(100, 1500/8000);
+  octave:171> save_array_c_header(h,"filtP650S900","t.h")
+*/
+
+float filtP650S900[100]={
+  -0.0003447438107989,
+  -0.0000773443016821,
+   0.0002396774493908,
+   0.0005183973970710,
+   0.0006638187313266,
+   0.0005975981831705,
+   0.0002897687607439,
+  -0.0002119524114060,
+  -0.0007715254253800,
+  -0.0011873172727259,
+  -0.0012521109061848,
+  -0.0008371281604995,
+   0.0000296985029260,
+   0.0011281958214742,
+   0.0020819312900066,
+   0.0024705948613571,
+   0.0019912318935963,
+   0.0006124281900638,
+  -0.0013451726955418,
+  -0.0032531555468084,
+  -0.0043549616066416,
+  -0.0040377485154437,
+  -0.0021065223027673,
+   0.0010488136629895,
+   0.0044817586852605,
+   0.0069407381267179,
+   0.0072892156498907,
+   0.0049639277235708,
+   0.0003023755700798,
+  -0.0054092967401810,
+  -0.0102368927453788,
+  -0.0121970459192933,
+  -0.0099848952803716,
+  -0.0035856433622624,
+   0.0054724953549814,
+   0.0143763043146873,
+   0.0197915861224756,
+   0.0189508548007168,
+   0.0107444584473630,
+  -0.0035511748160051,
+  -0.0202026606441822,
+  -0.0337226677294633,
+  -0.0382279329229346,
+  -0.0291127865967771,
+  -0.0045659298430227,
+   0.0335371151119154,
+   0.0796831473314644,
+   0.1257715935323888,
+   0.1629350232213685,
+   0.1836638608187552,
+   0.1836638608187552,
+   0.1629350232213685,
+   0.1257715935323888,
+   0.0796831473314644,
+   0.0335371151119154,
+  -0.0045659298430227,
+  -0.0291127865967771,
+  -0.0382279329229346,
+  -0.0337226677294633,
+  -0.0202026606441822,
+  -0.0035511748160051,
+   0.0107444584473630,
+   0.0189508548007168,
+   0.0197915861224756,
+   0.0143763043146873,
+   0.0054724953549814,
+  -0.0035856433622624,
+  -0.0099848952803716,
+  -0.0121970459192933,
+  -0.0102368927453788,
+  -0.0054092967401810,
+   0.0003023755700798,
+   0.0049639277235708,
+   0.0072892156498907,
+   0.0069407381267179,
+   0.0044817586852605,
+   0.0010488136629895,
+  -0.0021065223027673,
+  -0.0040377485154437,
+  -0.0043549616066416,
+  -0.0032531555468084,
+  -0.0013451726955418,
+   0.0006124281900638,
+   0.0019912318935963,
+   0.0024705948613571,
+   0.0020819312900066,
+   0.0011281958214742,
+   0.0000296985029260,
+  -0.0008371281604995,
+  -0.0012521109061848,
+  -0.0011873172727259,
+  -0.0007715254253800,
+  -0.0002119524114060,
+   0.0002897687607439,
+   0.0005975981831705,
+   0.0006638187313266,
+   0.0005183973970710,
+   0.0002396774493908,
+  -0.0000773443016821,
+  -0.0003447438107989
+};
+
+/*
+  Low pass filter, sample rate 8000 Hz, 60dB dB atten, pass 900 Hz, stop 1100 Hz
+  Initially used to build 700E BPF.
+
+  Generated using Octave:
+  octave:170> h = fir1(100, 2000/8000);
+  octave:171> save_array_c_header(h,"filtP900S1100","t.h")
+*/
+
+float filtP900S1100[]={
+   0.0004418158615696,
+   0.0001287637146300,
+  -0.0002887027062308,
+  -0.0005878482226797,
+  -0.0005730851022636,
+  -0.0001872428398401,
+   0.0004164064769556,
+   0.0009044243294414,
+   0.0009214286447099,
+   0.0003198823196113,
+  -0.0006735180958441,
+  -0.0014968764631324,
+  -0.0015397255009494,
+  -0.0005518542214485,
+   0.0010799820573451,
+   0.0024224384337026,
+   0.0024915418423841,
+   0.0009146857642078,
+  -0.0016601832557821,
+  -0.0037544126485350,
+  -0.0038606363852215,
+  -0.0014507340945908,
+   0.0024489243815523,
+   0.0055987186471211,
+   0.0057705307663451,
+   0.0022226683344412,
+  -0.0035036863386865,
+  -0.0081289422623303,
+  -0.0084269758191975,
+  -0.0033347717101330,
+   0.0049319056855186,
+   0.0116673991980949,
+   0.0122195177108518,
+   0.0049860194363942,
+  -0.0069600525064408,
+  -0.0169004061958973,
+  -0.0180041048574631,
+  -0.0076258897964686,
+   0.0101458756070637,
+   0.0255847362406635,
+   0.0280993703146360,
+   0.0125508957876736,
+  -0.0162678073198399,
+  -0.0438597596479416,
+  -0.0516282045539403,
+  -0.0256831488498926,
+   0.0353080736910379,
+   0.1173458947603512,
+   0.1953111386765199,
+   0.2427155307119274,
+   0.2427155307119275,
+   0.1953111386765199,
+   0.1173458947603512,
+   0.0353080736910379,
+  -0.0256831488498926,
+  -0.0516282045539403,
+  -0.0438597596479416,
+  -0.0162678073198399,
+   0.0125508957876736,
+   0.0280993703146360,
+   0.0255847362406635,
+   0.0101458756070637,
+  -0.0076258897964686,
+  -0.0180041048574631,
+  -0.0169004061958973,
+  -0.0069600525064408,
+   0.0049860194363942,
+   0.0122195177108518,
+   0.0116673991980949,
+   0.0049319056855186,
+  -0.0033347717101330,
+  -0.0084269758191975,
+  -0.0081289422623303,
+  -0.0035036863386865,
+   0.0022226683344412,
+   0.0057705307663451,
+   0.0055987186471211,
+   0.0024489243815523,
+  -0.0014507340945908,
+  -0.0038606363852214,
+  -0.0037544126485350,
+  -0.0016601832557821,
+   0.0009146857642078,
+   0.0024915418423841,
+   0.0024224384337026,
+   0.0010799820573451,
+  -0.0005518542214485,
+  -0.0015397255009494,
+  -0.0014968764631324,
+  -0.0006735180958441,
+   0.0003198823196113,
+   0.0009214286447099,
+   0.0009044243294414,
+   0.0004164064769556,
+  -0.0001872428398401,
+  -0.0005730851022636,
+  -0.0005878482226797,
+  -0.0002887027062308,
+   0.0001287637146300,
+   0.0004418158615696
+};
+
 // FIR filter suitable for changing rates 7500 to/from 8000
 // Sample 120000 Hz, pass 2700, stop 3730, ripple 0.1dB, atten 100 dB.  Stop 0.03108.
-float quiskFilt120t480[480] = { -0.000005050567303837, -0.000000267011791999, 0.000000197734700398, 0.000001038946634000,
+float quiskFilt120t480[480] = {
+-0.000005050567303837, -0.000000267011791999, 0.000000197734700398, 0.000001038946634000,
  0.000002322193058869, 0.000004115682735322, 0.000006499942123311, 0.000009551098482930, 0.000013350669444763,
  0.000017966192635412, 0.000023463361155584, 0.000029885221425020, 0.000037271082107518, 0.000045630720487935,
  0.000054970017069384, 0.000065233162392019, 0.000076360900545177, 0.000088271373315159, 0.000100818605854714,
@@ -159,4 +500,3 @@ float quiskFilt120t480[480] = { -0.000005050567303837, -0.000000267011791999, 0.
  0.000023463361155584, 0.000017966192635412, 0.000013350669444763, 0.000009551098482930, 0.000006499942123311,
  0.000004115682735322, 0.000002322193058869, 0.000001038946634000, 0.000000197734700398, -0.000000267011791999,
 -0.000005050567303837 };
-
