@@ -40,8 +40,6 @@
 #include "mbest.h"
 #include "newamp1.h"
 
-#define NEWAMP1_VQ_MBEST_DEPTH 5  /* how many candidates we keep for each stage of mbest search */
-
 /*---------------------------------------------------------------------------*\
 
   FUNCTION....: interp_para()
@@ -189,7 +187,6 @@ float rate_K_mbest_encode(int *indexes, float *x, float *xq, int ndim, int mbest
   /* Stage 1 */
 
   mbest_search(codebook1, x, w, ndim, newamp1vq_cb[0].m, mbest_stage1, index);
-  MBEST_PRINT("Stage 1:", mbest_stage1);
 
   /* Stage 2 */
 
@@ -199,7 +196,6 @@ float rate_K_mbest_encode(int *indexes, float *x, float *xq, int ndim, int mbest
 	  target[i] = x[i] - codebook1[ndim*n1+i];
       mbest_search(codebook2, target, w, ndim, newamp1vq_cb[1].m, mbest_stage2, index);
   }
-  MBEST_PRINT("Stage 2:", mbest_stage2);
 
   n1 = mbest_stage2->list[0].index[1];
   n2 = mbest_stage2->list[0].index[0];
@@ -242,7 +238,7 @@ void post_filter_newamp1(float vec[], float sample_freq_kHz[], int K, float pf_g
       vec is rate K vector describing spectrum of current frame lets
       pre-emp before applying PF. 20dB/dec over 300Hz.  Postfilter
       affects energy of frame so we measure energy before and after
-      and normalise.  Plenty of room for experiment here as well.
+      and normalise.  Plenty of room for experimentation here.
     */
     
     float pre[K];
@@ -340,8 +336,7 @@ void resample_rate_L(C2CONST *c2const, MODEL *model, float rate_K_vec[], float r
 
    for(k=0; k<K; k++) {
        rate_K_vec_term[k+1] = rate_K_vec[k];
-       rate_K_sample_freqs_kHz_term[k+1] = rate_K_sample_freqs_kHz[k];
-  
+       rate_K_sample_freqs_kHz_term[k+1] = rate_K_sample_freqs_kHz[k];  
        //printf("k: %d f: %f rate_K: %f\n", k, rate_K_sample_freqs_kHz[k], rate_K_vec[k]);
    }
 
@@ -394,8 +389,69 @@ void determine_phase(C2CONST *c2const, COMP H[], MODEL *model, int Nfft, codec2_
     }
 }
 
+/*---------------------------------------------------------------------------* \
 
-/*---------------------------------------------------------------------------*\
+  FUNCTION....: determine_autoc
+  AUTHOR......: David Rowe
+  DATE CREATED: April 2020
+
+  Determine autocorrelation coefficients from model params, for machine 
+  learning experiments.
+
+\*---------------------------------------------------------------------------*/
+
+void determine_autoc(C2CONST *c2const, float Rk[], int order, MODEL *model, int Nfft, codec2_fft_cfg fwd_cfg, codec2_fft_cfg inv_cfg)
+{
+    int i,m;
+    int Ns = Nfft/2+1;
+    float Gdbfk[Ns], sample_freqs_kHz[Ns];
+    float AmdB[MAX_AMP+1], rate_L_sample_freqs_kHz[MAX_AMP+1];
+
+    /* interpolate in the log domain */
+    for(m=1; m<=model->L; m++) {
+        assert(model->A[m] != 0.0);
+        AmdB[m] = 20.0*log10f(model->A[m]);
+        rate_L_sample_freqs_kHz[m] = (float)m*model->Wo*(c2const->Fs/2000.0)/M_PI;        
+    }
+    
+    for(i=0; i<Ns; i++) {
+        sample_freqs_kHz[i] = (c2const->Fs/1000.0)*(float)i/Nfft;
+    }
+
+    interp_para(Gdbfk, &rate_L_sample_freqs_kHz[1], &AmdB[1], model->L, sample_freqs_kHz, Ns);
+
+    COMP S[Nfft], R[Nfft];
+
+    /* install negative frequency components, convert to mag squared of spectrum */
+    S[0].real = pow(10.0, Gdbfk[0]/10.0);
+    S[0].imag = 0.0;
+    for(i=1; i<Ns; i++) {
+	S[i].real = S[Nfft-i].real = pow(10.0, Gdbfk[i]/10.0);
+	S[i].imag = S[Nfft-i].imag = 0.0;
+    }
+
+    /* IDFT of mag squared is autocorrelation function */
+    codec2_fft(inv_cfg, S, R);
+    for(int k=0; k<order+1; k++) Rk[k] = R[k].real;
+}
+
+
+/* update and optionally run "front eq" equaliser on before VQ */
+void newamp1_eq(float rate_K_vec_no_mean[], float eq[], int K, int eq_en) {
+    static float ideal[] = {8,10,12,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,-20};
+    float gain = 0.02;
+    float update;
+    
+    for(int k=0; k<K; k++) {
+        update = rate_K_vec_no_mean[k] - ideal[k];
+        eq[k] = (1.0-gain)*eq[k] + gain*update;
+        if (eq[k] < 0.0) eq[k] = 0.0;
+        if (eq_en)
+            rate_K_vec_no_mean[k] -= eq[k];
+    }
+}
+
+/*---------------------------------------------------------------------------* \
 
   FUNCTION....: newamp1_model_to_indexes
   AUTHOR......: David Rowe
@@ -434,24 +490,14 @@ void newamp1_model_to_indexes(C2CONST *c2const,
         rate_K_vec_no_mean[k] = rate_K_vec[k] - *mean;
 
     /* update and optionally run "front eq" equaliser on before VQ */
-    static float ideal[] = {8,10,12,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,-20};
-    float gain = 0.02;
-    float update;
-        
-    for(k=0; k<K; k++) {
-        update = rate_K_vec_no_mean[k] - ideal[k];
-        eq[k] = (1.0-gain)*eq[k] + gain*update;
-        if (eq[k] < 0.0) eq[k] = 0.0;
-        if (eq_en)
-            rate_K_vec_no_mean[k] -= eq[k];
-    }
-
+    newamp1_eq(rate_K_vec_no_mean, eq, K, eq_en);
+ 
     /* two stage VQ */
     rate_K_mbest_encode(indexes, rate_K_vec_no_mean, rate_K_vec_no_mean_, K, NEWAMP1_VQ_MBEST_DEPTH);
 
     /* running sum of squared error for variance calculation */
     for(k=0; k<K; k++)
-        *se += pow(rate_K_vec_no_mean[k]-rate_K_vec_no_mean_[k],2.0);
+        *se += (float)pow(rate_K_vec_no_mean[k]-rate_K_vec_no_mean_[k],2.0);
 
     /* scalar quantise mean (effectively the frame energy) */
     float w[1] = {1.0};
@@ -475,8 +521,7 @@ void newamp1_model_to_indexes(C2CONST *c2const,
     else {
         indexes[3] = 0;
     }
-
- }
+}
 
 
 /*---------------------------------------------------------------------------*\
@@ -615,7 +660,7 @@ void newamp1_indexes_to_model(C2CONST *c2const,
 
     interp_Wo_v(aWo_, aL_, avoicing_, *Wo_left, Wo_right, *voicing_left, voicing_right);
 
-    /* back to rate L amplitudes, synthesis phase for each frame */
+    /* back to rate L amplitudes, synthesise phase for each frame */
 
     for(i=0; i<M; i++) {
         model_[i].Wo = aWo_[i];
