@@ -1,9 +1,8 @@
 #!/bin/bash -x
-# 
+#
 # David Sep 2019
 # Tests 2020 OFDM modem fading channel performance in DPSK mode, using a simulated faster (2Hz) high SNR fading channel
 
-PATH=$PATH:../build_linux/src
 RAW=$PWD/../raw
 results=$(mktemp)
 
@@ -16,9 +15,11 @@ if [ ! -f ../raw/faster_fading_samples.float ]; then
 fi
 
 pwd
+
 # Coded BER should be < 1% for this test
-nc=31; ofdm_mod --in /dev/zero --testframes 300 --nc $nc --ldpc 2 --verbose 1 -p 312 --dpsk | cohpsk_ch - - -40 --Fs 8000 -f 10 --ssbfilt 1 --faster --raw_dir $RAW | ofdm_demod --out /dev/null --testframes --nc $nc --verbose 1 --ldpc 2 -p 312 --dpsk 2> $results
+ofdm_mod --in /dev/zero --testframes 300 --mode 2020 --ldpc --verbose 1 -p 312 --dpsk | \
+cohpsk_ch - - -40 --Fs 8000 -f 10 --ssbfilt 1 --mpd --raw_dir $RAW --multipath_delay 2 | \
+ofdm_demod --out /dev/null --testframes --mode 2020 --verbose 1 --ldpc -p 312 --dpsk 2> $results
 cat $results
 cber=$(cat $results | sed -n "s/^Coded BER.* \([0-9..]*\) Tbits.*/\1/p")
 python -c "import sys; sys.exit(0) if $cber<=0.05 else sys.exit(1)"
-
